/*
 * Multi Commander - SDK
 * 
 * Copyright (C) 2000-2016 All Rights Reserved , http://multicommander.com
 * =======================================================================================
 * 
 * Changes
 * ---------------------
 * 
 */

#pragma once

#include "MCNamespace.h"
#include "MultiApp_Def.h"
MCNSBEGIN

// Attributes flags for FileSystem item ( 0x00010000L and below is same as Windows FILE_ATTRIBUTE_xxx flags )
#define ZFF_READONLY        0x00000001L // Item has ReadOnly Attribute set
#define ZFF_HIDDEN          0x00000002L // Item has Hidden Attribute Set
#define ZFF_SYSTEM          0x00000004L // Item has System attribute Set
#define ZFF_NOTUSED         0x00000008L // Windows do not have any for this anymore
#define ZFF_FOLDER          0x00000010L // Item is a Folder
#define ZFF_ARCHIVE         0x00000020L // Item has Archive Attribute set

#define ZFF_DEVICE          0x00000040L // FILE_ATTRIBUTE_DEVICE
#define ZFF_NORMAL          0x00000080L // FILE_ATTRIBUTE_NORMAL
#define ZFF_TEMPORARY       0x00000100L //
#define ZFF_SPARSE_FILE     0x00000200L //
#define ZFF_REPARSE_POINT   0x00000400L // Junction or Symlink or Mounted Path
#define ZFF_COMPRESSED      0x00000800L // File is compressed
#define ZFF_OFFLINE         0x00001000L // File is offline - Reading it will be very slow since filesystem will go out to backup system and read it
#define ZFF_NOTINDEXED      0x00002000L // FILE_ATTRIBUTE_NOT_CONTENT_INDEXED 
#define ZFF_ENCRYPTED       0x00004000L // File is encrypted 
#define ZFF_VIRTUAL         0x00010000L // File is virtual (Reserved by windows but not used)

// MC Extended file attributes Flags ( NOT same as Windows internal )
#define ZFF_PLACEHOLDER     0x00020000L // Placeholder file are fake files with have a size but do not actually take up any room. (New from Win8.1, Used in Win8.1 for Skydrive/OneDirve folder)
#define ZFF_SHORTCUT        0x00040000L // Item is a .lnk shortcut file
#define ZFF_SHORTCUTFOLDER  0x00080000L // Shortcut file links to a folder.
#define ZFF_SERVER          0x00100000L // Item is a Network Server 
#define ZFF_SHARE_DRIVE     0x00200000L // Item is a Network Share (Drive)
#define ZFF_MCLINK          0x00400000L // Item is a Virtual Item. (Does not exist on disk) But the item contain a path in Extended PropData (0, EXTPROP_LINKTARGET )
#define ZFF_VOLCOMMAND      0x00800000L // Item is a special volume commands. (Special handling)
#define ZFF_HIGHLIGHT       0x01000000L // File Item is to be highlighted. Show as BOLD ? or Highlight color 
#define ZFF_SYMLINK         0x02000000L // File item is a SymLink ( check Reserv0 for IS_REPARSE_TAG_SYMLINK )
#define ZFF_JUNKTION        0x04000000L // File Item is a Junction
#define ZFF_MOUNTPOINT      0x08000000L // File Item is a Junction to a Device (Device Mounted in a folder )
#define ZFF_DISCONNECT      0x10000000L // Item can be Disconnected. Eg Items exists on FTP/CDROM/USB Device (require the volume also support IVF_CONNECT )
#define ZFF_NOFILE          0x20000000L // This file item is not a file. So do not split names with dot into a file extension part (Eg Registry Values)
#define ZFF_CANNOTDELETE    0x40000000L // Item can not be deleted. (Can be a fake item, like FSFTP "Quick connect" 'file')
#define ZFF_RESERVED        0x80000000L // Not used

#define ZFF_MC_EXTENDED     0xFFFF0000L // Multi Commander Attribute. Use "dwAttributes ^= dwAttributes & ZFF_MC_EXTENDED;" to remove all MC attributes.

// ZFileitem Update Flags, returned by Set_Filedata
#define ZFU_NAME   0x00001 // Name is updated.. ( Not used , can't update name )
#define ZFU_SIZE   0x00002 // size has changed
#define ZFU_ATTRIB 0x00004 // Attrib has changed
#define ZFU_DATE   0x00008 // Date is changed

// ExtraProp DataType ID  // BYTE   TypeID;
#define ZFXP_STRING      0x01
#define ZFXP_BYTEDATA    0x02
#define ZFXP_VAL32       0x04  // value prop
#define ZFXP_VAL64       0x08  // value prop
#define ZFXP_VOIDPTR     0x10 
#define ZFXP_FILETIME    0x20  // value prop - You can only set 1 Of the value props in the same property
#define ZFXP_DOUBLE      0x40  // value prop is double
#define ZFXP_DISPLAYNAME 0x80  // DisplayName is set

// Extended property for fileitem with attribute ZFF_MCLINK
// GetExtraPropData(0, ZEXTPROP_LINKTARGET)
#define ZEXTPROP_LINKTARGET 42
#define ZEXTPROP_LINKTARGET_TYPE_PATH 1
#define ZEXTPROP_LINKTARGET_TYPE_URL  2

#define ZF_TAG1   0x01
#define ZF_TAG2   0x02
#define ZF_TAG3   0x04
#define ZF_TAG4   0x08
#define ZF_TAG5   0x10
#define ZF_TAG6   0x20
#define ZF_TAG7   0x40
#define ZF_TAG8   0x80

// Extended FileProperty Data 
struct ExtraProp
{
  BYTE   Flag;

  WCHAR  szDisplayName[512];

  /* change to
  union
  {
    void* pPtr;
    const wchar_t* pText;
  } pData;
  */
  const void* pPtr;
  DWORD  nDataLen;  // In GetProp. this is the LEN of pPtr

  union
  {
    INT64  value64;
    double valueD; // Flag must have ZFXP_DOUBLE
  };
};

class __declspec(novtable) IFileItem
{
public:
  virtual WCHAR* Get_Name( WCHAR* szName , DWORD nMaxLen ) const = 0;  
  virtual WCHAR* Get_NameOnly(WCHAR* szName, DWORD nMaxLen) const = 0;
  virtual WCHAR* Get_Path( WCHAR* szName , DWORD nMaxLen ) const = 0;
  virtual WCHAR* Get_FullPath( WCHAR* szName, DWORD nMaxLen ) const = 0;

  // The the name of the parent item
  virtual WCHAR* Get_ParentName(WCHAR* szName, DWORD nMaxLen) const = 0;

  virtual WCHAR* Get_Comment( WCHAR* strComment ) = 0;
  virtual WCHAR* Get_Ext( WCHAR* strExt,BOOL bStripDot = FALSE  ) const = 0;
  virtual const WCHAR* Get_Ext(BOOL bStripDot = FALSE) const = 0;

  virtual INT64  Get_Size() const = 0;
  virtual DWORD  Get_Attributes() const = 0;

  virtual int   Get_IconIdx(MCIconSize::MCIconSize nIconSize) const = 0;
  virtual int   Get_IconNormalSmall() const = 0;
  virtual int   Get_IconNormalLarge() const = 0;
  virtual int   Get_IconNormalMedium() const = 0;
  virtual int   Get_IconNormalXLarge() const = 0;

  virtual  UINT8  ColorID() const = 0;

  virtual void  Get_Date(FILETIME* ft) const = 0;

  virtual void  Get_DateAccess(FILETIME* ft) const = 0;
  virtual void  Get_DateWrite(FILETIME* ft) const = 0;
  virtual void  Get_DateCreate(FILETIME* ft) const = 0;

  virtual bool  isFolder() const = 0;
  virtual bool  isReadOnly() const = 0;
  virtual bool  isHidden() const = 0;
  virtual bool  isSystem() const = 0;
  virtual bool  isArchive() const = 0;

  virtual bool  isDotFolder() const = 0;

  virtual DWORD Set_FileData( const WCHAR* strName , const WCHAR* strComment ,  INT64 i64Size , DWORD dwAttribute , const FILETIME* pftWriteTime, const FILETIME* pftCreateTime, const FILETIME* pftAccessTime ) = 0;
  virtual BOOL  Set_FindData( WIN32_FIND_DATA* pFindData ) = 0;

  virtual  short GetVolumeID() const = 0;

  // IF true then this item is about to be deleted. (The c++ object, not file)
  virtual BOOL  HasFlagDelete() const = 0;

  // if true this item will be tagged as being on a remote volume (eg ftp ) 
  // This item is located on a volume that is SLOW with long latency. 
  virtual void  SetIsRemote(bool b) = 0;
  virtual BOOL  IsRemote() = 0;

  virtual UINT64 GetHashID() = 0; // Need to be changed. 64bit value not big enough for unique data, Can generate conflicting IDs.

  // set to true if browsing this path should be redirected to a NumDevice ( eg 6:\ )
  virtual void SetRedirectToNumDevice(bool b) = 0;

  // Set Extra property info that you want to store inside the FileItem object.
  // Can be used when getting property with a property info extensions. 
  virtual BOOL  SetExtraPropData(WORD nExtID, WORD nPropID, ExtraProp* pProp) = 0;
  virtual BOOL  GetExtraPropData(WORD nExtID, WORD nPropID, ExtraProp* pProp) const = 0;

  // if set to true then extra prop data will be fetched again. even if it is already cached.
  virtual void  SetExtraPropDataInvalid(bool b) = 0;

  virtual IFileItem* GetParentItem() = 0;

  //virtual bool GetLinkTarget(WCHAR* szTarget, DWORD nLen) = 0;

  // bool bRecursive parameter obsolete
  virtual void AddRef(bool bRecursive = false) = 0;
  virtual void DelRef(bool bRecursive = false) = 0;
  virtual DWORD GetRef() = 0;
};


struct FileItem
{
  DWORD lAttribute;   // ZFF_ Flags

  ULARGE_INTEGER lSize; // unsigned long

  FILETIME  tCreate;  // Can be 0 - on FAT file system this is not known
  FILETIME  tAccess;  // Can be 0 - on FAT file system this is not known
  FILETIME  tWrite;   //

  WCHAR   wName[1024];  //We have a virtual filesystem here and we must allow bigger name parts then 256
  int     iIconSmall;
  int     iIconLarge;
  int     iIconMedium;
  int     iIconXLarge;
  DWORD_PTR dwData;   // Custom Data that the plugin can put there..  this data is returned if used select item
                      // ***** WARRNING!!! *********
                      // Since you can have multiple instances of a modules/plugin and you can not be sure that an item you get is from 
                      // the same instance that created the item. so be careful of what data you put here
};

MCNSEND
